@@ -11,8 +11,8 @@ import (
 
 	"filippo.io/age"
 	"filippo.io/age/armor"
-	"go.uber.org/multierr"
 
+	"github.com/twpayne/chezmoi/v2/internal/chezmoierrors"
 	"github.com/twpayne/chezmoi/v2/internal/chezmoilog"
 )
 
@@ -251,7 +251,7 @@ func parseIdentityFile(identityFile AbsPath) (identities []age.Identity, err err
 	if file, err = os.Open(identityFile.String()); err != nil {
 		return
 	}
-	defer multierr.AppendInvoke(&err, multierr.Close(file))
+	defer chezmoierrors.CombineFunc(&err, file.Close)
 	identities, err = age.ParseIdentities(file)
 	return
 }
@@ -263,7 +263,7 @@ func parseRecipientsFile(recipientsFile AbsPath) (recipients []age.Recipient, er
 	if file, err = os.Open(recipientsFile.String()); err != nil {
 		return
 	}
-	defer multierr.AppendInvoke(&err, multierr.Close(file))
+	defer chezmoierrors.CombineFunc(&err, file.Close)
 	recipients, err = age.ParseRecipients(file)
 	return
 }
