@@ -3,6 +3,7 @@ package chezmoi
 import (
 	"io/fs"
 	"os/exec"
+	"time"
 
 	"github.com/rs/zerolog"
 	vfs "github.com/twpayne/go-vfs/v4"
@@ -24,6 +25,17 @@ func NewDebugSystem(system System, logger *zerolog.Logger) *DebugSystem {
 	}
 }
 
+// Chtimes implements System.Chtimes.
+func (s *DebugSystem) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	err := s.system.Chtimes(name, atime, mtime)
+	s.logger.Err(err).
+		Stringer("name", name).
+		Time("atime", atime).
+		Time("mtime", mtime).
+		Msg("Chtimes")
+	return err
+}
+
 // Chmod implements System.Chmod.
 func (s *DebugSystem) Chmod(name AbsPath, mode fs.FileMode) error {
 	err := s.system.Chmod(name, mode)
@@ -44,28 +56,6 @@ func (s *DebugSystem) Glob(name string) ([]string, error) {
 	return matches, err
 }
 
-// IdempotentCmdCombinedOutput implements System.IdempotentCmdCombinedOutput.
-func (s *DebugSystem) IdempotentCmdCombinedOutput(cmd *exec.Cmd) ([]byte, error) {
-	output, err := s.system.IdempotentCmdCombinedOutput(cmd)
-	s.logger.Err(err).
-		EmbedObject(chezmoilog.OSExecCmdLogObject{Cmd: cmd}).
-		Bytes("output", chezmoilog.Output(output, err)).
-		EmbedObject(chezmoilog.OSExecExitErrorLogObject{Err: err}).
-		Msg("IdempotentCmdCombinedOutput")
-	return output, err
-}
-
-// IdempotentCmdOutput implements System.IdempotentCmdOutput.
-func (s *DebugSystem) IdempotentCmdOutput(cmd *exec.Cmd) ([]byte, error) {
-	output, err := s.system.IdempotentCmdOutput(cmd)
-	s.logger.Err(err).
-		EmbedObject(chezmoilog.OSExecCmdLogObject{Cmd: cmd}).
-		Bytes("output", chezmoilog.Output(output, err)).
-		EmbedObject(chezmoilog.OSExecExitErrorLogObject{Err: err}).
-		Msg("IdempotentCmdOutput")
-	return output, err
-}
-
 // Link implements System.Link.
 func (s *DebugSystem) Link(oldpath, newpath AbsPath) error {
 	err := s.system.Link(oldpath, newpath)
@@ -130,6 +120,15 @@ func (s *DebugSystem) Readlink(name AbsPath) (string, error) {
 	return linkname, err
 }
 
+// Remove implements System.Remove.
+func (s *DebugSystem) Remove(name AbsPath) error {
+	err := s.system.Remove(name)
+	s.logger.Err(err).
+		Stringer("name", name).
+		Msg("Remove")
+	return err
+}
+
 // RemoveAll implements System.RemoveAll.
 func (s *DebugSystem) RemoveAll(name AbsPath) error {
 	err := s.system.RemoveAll(name)
@@ -159,24 +158,20 @@ func (s *DebugSystem) RunCmd(cmd *exec.Cmd) error {
 	return err
 }
 
-// RunIdempotentCmd implements System.RunIdempotentCmd.
-func (s *DebugSystem) RunIdempotentCmd(cmd *exec.Cmd) error {
-	err := s.system.RunIdempotentCmd(cmd)
-	s.logger.Err(err).
-		EmbedObject(chezmoilog.OSExecCmdLogObject{Cmd: cmd}).
-		EmbedObject(chezmoilog.OSExecExitErrorLogObject{Err: err}).
-		Msg("RunIdempotentCmd")
-	return err
-}
-
 // RunScript implements System.RunScript.
-func (s *DebugSystem) RunScript(scriptname RelPath, dir AbsPath, data []byte, interpreter *Interpreter) error {
-	err := s.system.RunScript(scriptname, dir, data, interpreter)
+func (s *DebugSystem) RunScript(
+	scriptname RelPath,
+	dir AbsPath,
+	data []byte,
+	options RunScriptOptions,
+) error {
+	err := s.system.RunScript(scriptname, dir, data, options)
 	s.logger.Err(err).
 		Stringer("scriptname", scriptname).
 		Stringer("dir", dir).
 		Bytes("data", chezmoilog.Output(data, err)).
-		Object("interpreter", interpreter).
+		Object("interpreter", options.Interpreter).
+		Str("condition", string(options.Condition)).
 		EmbedObject(chezmoilog.OSExecExitErrorLogObject{Err: err}).
 		Msg("RunScript")
 	return err
@@ -196,6 +191,11 @@ func (s *DebugSystem) UnderlyingFS() vfs.FS {
 	return s.system.UnderlyingFS()
 }
 
+// UnderlyingSystem implements System.UnderlyingSystem.
+func (s *DebugSystem) UnderlyingSystem() System {
+	return s.system
+}
+
 // WriteFile implements System.WriteFile.
 func (s *DebugSystem) WriteFile(name AbsPath, data []byte, perm fs.FileMode) error {
 	err := s.system.WriteFile(name, data, perm)
