@@ -427,6 +427,23 @@ func TestSourceStateAdd(t *testing.T) {
 				),
 			},
 		},
+		{
+			name: "template",
+			destAbsPaths: []AbsPath{
+				NewAbsPath("/home/user/.template"),
+			},
+			addOptions: AddOptions{
+				AutoTemplate: true,
+				Filter:       NewEntryTypeFilter(EntryTypesAll, EntryTypesNone),
+			},
+			tests: []any{
+				vfst.TestPath("/home/user/.local/share/chezmoi/dot_template.tmpl",
+					vfst.TestModeIsRegular,
+					vfst.TestModePerm(0o666&^chezmoitest.Umask),
+					vfst.TestContentsString("key = {{ .variable }}\n"),
+				),
+			},
+		},
 		{
 			name: "dir_and_dir_file",
 			destAbsPaths: []AbsPath{
@@ -1929,7 +1946,7 @@ func (s *SourceState) applyAll(
 ) error {
 	for _, targetRelPath := range s.TargetRelPaths() {
 		switch err := s.Apply(targetSystem, destSystem, persistentState, targetDirAbsPath, targetRelPath, options); {
-		case errors.Is(err, Skip):
+		case errors.Is(err, fs.SkipDir):
 			continue
 		case err != nil:
 			return err
