@@ -2,6 +2,7 @@ package cmd
 
 import (
 	"fmt"
+	"io/fs"
 
 	"github.com/spf13/cobra"
 
@@ -10,6 +11,7 @@ import (
 
 type addCmdConfig struct {
 	TemplateSymlinks bool `json:"templateSymlinks" mapstructure:"templateSymlinks" yaml:"templateSymlinks"`
+	autoTemplate     bool
 	create           bool
 	encrypt          bool
 	exact            bool
@@ -39,6 +41,13 @@ func (c *Config) newAddCmd() *cobra.Command {
 	}
 
 	flags := addCmd.Flags()
+	flags.BoolVarP(
+		&c.Add.autoTemplate,
+		"autotemplate",
+		"a",
+		c.Add.autoTemplate,
+		"Generate the template when adding files as templates",
+	) //nolint:lll
 	flags.BoolVar(
 		&c.Add.create,
 		"create",
@@ -98,7 +107,7 @@ func (c *Config) defaultPreAddFunc(targetRelPath chezmoi.RelPath) error {
 			c.Add.prompt = false
 			return nil
 		case choice == "no":
-			return chezmoi.Skip
+			return fs.SkipDir
 		case choice == "quit":
 			return chezmoi.ExitCodeError(0)
 		case choice == "yes":
@@ -153,7 +162,7 @@ func (c *Config) defaultReplaceFunc(
 			c.force = true
 			return nil
 		case choice == "no":
-			return chezmoi.Skip
+			return fs.SkipDir
 		case choice == "quit":
 			return chezmoi.ExitCodeError(0)
 		case choice == "yes":
@@ -188,6 +197,7 @@ func (c *Config) runAddCmd(
 		c.destSystem,
 		destAbsPathInfos,
 		&chezmoi.AddOptions{
+			AutoTemplate:    c.Add.autoTemplate,
 			Create:          c.Add.create,
 			Encrypt:         c.Add.encrypt,
 			EncryptedSuffix: c.encryption.EncryptedSuffix(),
