@@ -40,7 +40,6 @@ import (
 	"github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-xdg/v6"
 	cobracompletefig "github.com/withfig/autocomplete-tools/integrations/cobra"
-	"go.uber.org/multierr"
 	"golang.org/x/exp/maps"
 	"golang.org/x/exp/slices"
 	"golang.org/x/term"
@@ -49,6 +48,7 @@ import (
 
 	"github.com/twpayne/chezmoi/v2/assets/templates"
 	"github.com/twpayne/chezmoi/v2/internal/chezmoi"
+	"github.com/twpayne/chezmoi/v2/internal/chezmoierrors"
 	"github.com/twpayne/chezmoi/v2/internal/chezmoilog"
 	"github.com/twpayne/chezmoi/v2/internal/git"
 	"github.com/twpayne/chezmoi/v2/internal/shell"
@@ -410,6 +410,8 @@ func newConfig(options ...configOption) (*Config, error) {
 		"gitHubKeys":               c.gitHubKeysTemplateFunc,
 		"gitHubLatestRelease":      c.gitHubLatestReleaseTemplateFunc,
 		"gitHubLatestTag":          c.gitHubLatestTagTemplateFunc,
+		"gitHubReleases":           c.gitHubReleasesTemplateFunc,
+		"gitHubTags":               c.gitHubTagsTemplateFunc,
 		"glob":                     c.globTemplateFunc,
 		"gopass":                   c.gopassTemplateFunc,
 		"gopassRaw":                c.gopassRawTemplateFunc,
@@ -486,16 +488,16 @@ func newConfig(options ...configOption) (*Config, error) {
 
 // Close closes resources associated with c.
 func (c *Config) Close() error {
-	var err error
+	errs := make([]error, 0, len(c.tempDirs))
 	for _, tempDirAbsPath := range c.tempDirs {
-		err2 := os.RemoveAll(tempDirAbsPath.String())
-		c.logger.Err(err2).
+		err := os.RemoveAll(tempDirAbsPath.String())
+		c.logger.Err(err).
 			Stringer("tempDir", tempDirAbsPath).
 			Msg("RemoveAll")
-		err = multierr.Append(err, err2)
+		errs = append(errs, err)
 	}
 	pprof.StopCPUProfile()
-	return err
+	return chezmoierrors.Combine(errs...)
 }
 
 // addTemplateFunc adds the template function with the given key and value
@@ -613,7 +615,7 @@ func (c *Config) applyArgs(
 		switch err := sourceState.Apply(
 			targetSystem, c.destSystem, c.persistentState, targetDirAbsPath, targetRelPath, applyOptions,
 		); {
-		case errors.Is(err, chezmoi.Skip):
+		case errors.Is(err, fs.SkipDir):
 			continue
 		case err != nil && c.keepGoing:
 			c.errorf("%v\n", err)
@@ -771,6 +773,8 @@ func (c *Config) createConfigFile(
 		"exit":             c.exitInitTemplateFunc,
 		"promptBool":       c.promptBoolInteractiveTemplateFunc,
 		"promptBoolOnce":   c.promptBoolOnceInteractiveTemplateFunc,
+		"promptChoice":     c.promptChoiceInteractiveTemplateFunc,
+		"promptChoiceOnce": c.promptChoiceOnceInteractiveTemplateFunc,
 		"promptInt":        c.promptIntInteractiveTemplateFunc,
 		"promptIntOnce":    c.promptIntOnceInteractiveTemplateFunc,
 		"promptString":     c.promptStringInteractiveTemplateFunc,
@@ -954,7 +958,7 @@ func (c *Config) defaultPreApplyFunc(
 			case choice == "yes":
 				return nil
 			case choice == "no":
-				return chezmoi.Skip
+				return fs.SkipDir
 			case choice == "all":
 				c.interactive = false
 				return nil
@@ -1003,7 +1007,7 @@ func (c *Config) defaultPreApplyFunc(
 			c.force = true
 			return nil
 		case choice == "skip":
-			return chezmoi.Skip
+			return fs.SkipDir
 		case choice == "quit":
 			return chezmoi.ExitCodeError(0)
 		default:
@@ -1395,6 +1399,7 @@ func (c *Config) gitAutoCommit(status *git.Status) error {
 	funcMap := maps.Clone(sprig.TxtFuncMap())
 	maps.Copy(funcMap, map[string]any{
 		"promptBool":   c.promptBoolInteractiveTemplateFunc,
+		"promptChoice": c.promptChoiceInteractiveTemplateFunc,
 		"promptInt":    c.promptIntInteractiveTemplateFunc,
 		"promptString": c.promptStringInteractiveTemplateFunc,
 		"targetRelPath": func(source string) string {
@@ -1516,7 +1521,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		"Specify targets by source path",
 	)
 
-	if err := multierr.Combine(
+	if err := chezmoierrors.Combine(
 		rootCmd.MarkPersistentFlagFilename("config"),
 		rootCmd.MarkPersistentFlagFilename("cpu-profile"),
 		persistentFlags.MarkHidden("cpu-profile"),
