@@ -5,7 +5,6 @@ import (
 	"fmt"
 	"io/fs"
 	"regexp"
-	"runtime"
 	"strconv"
 
 	"github.com/go-git/go-git/v5"
@@ -19,71 +18,70 @@ import (
 )
 
 type initCmdConfig struct {
-	apply       bool
-	branch      string
-	configPath  chezmoi.AbsPath
-	data        bool
-	depth       int
-	exclude     *chezmoi.EntryTypeSet
-	oneShot     bool
-	purge       bool
-	purgeBinary bool
-	ssh         bool
+	apply             bool
+	branch            string
+	configPath        chezmoi.AbsPath
+	data              bool
+	depth             int
+	filter            *chezmoi.EntryTypeFilter
+	guessRepoURL      bool
+	oneShot           bool
+	purge             bool
+	purgeBinary       bool
+	recurseSubmodules bool
+	ssh               bool
 }
 
-var dotfilesRepoGuesses = []struct {
-	rx                    *regexp.Regexp
-	httpRepoGuessRepl     string
-	httpUsernameGuessRepl string
-	sshRepoGuessRepl      string
+var repoGuesses = []struct {
+	rx                *regexp.Regexp
+	httpRepoGuessRepl string
+	sshRepoGuessRepl  string
 }{
 	{
-		rx:                    regexp.MustCompile(`\A([-0-9A-Za-z]+)\z`),
-		httpRepoGuessRepl:     "https://github.com/$1/dotfiles.git",
-		httpUsernameGuessRepl: "$1",
-		sshRepoGuessRepl:      "git@github.com:$1/dotfiles.git",
+		rx:                regexp.MustCompile(`\A([-0-9A-Za-z]+)\z`),
+		httpRepoGuessRepl: "https://github.com/$1/dotfiles.git",
+		sshRepoGuessRepl:  "git@github.com:$1/dotfiles.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\A([-0-9A-Za-z]+)/([-0-9A-Za-z]+)(\.git)?\z`),
-		httpRepoGuessRepl:     "https://github.com/$1/$2.git",
-		httpUsernameGuessRepl: "$1",
-		sshRepoGuessRepl:      "git@github.com:$1/$2.git",
+		rx:                regexp.MustCompile(`\A([-0-9A-Za-z]+)/([-\.0-9A-Z_a-z]+?)(\.git)?\z`),
+		httpRepoGuessRepl: "https://github.com/$1/$2.git",
+		sshRepoGuessRepl:  "git@github.com:$1/$2.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\A([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)\z`),
-		httpRepoGuessRepl:     "https://$1/$2/dotfiles.git",
-		httpUsernameGuessRepl: "$2",
-		sshRepoGuessRepl:      "git@$1:$2/dotfiles.git",
+		rx:                regexp.MustCompile(`\A([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)\z`),
+		httpRepoGuessRepl: "https://$1/$2/dotfiles.git",
+		sshRepoGuessRepl:  "git@$1:$2/dotfiles.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\A([-0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-.0-9A-Za-z]+)\z`),
-		httpRepoGuessRepl:     "https://$1/$2/$3.git",
-		httpUsernameGuessRepl: "$2",
-		sshRepoGuessRepl:      "git@$1:$2/$3.git",
+		rx: regexp.MustCompile(
+			`\A([-0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-.0-9A-Za-z]+)\z`,
+		),
+		httpRepoGuessRepl: "https://$1/$2/$3.git",
+		sshRepoGuessRepl:  "git@$1:$2/$3.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\A([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-0-9A-Za-z]+)(\.git)?\z`),
-		httpRepoGuessRepl:     "https://$1/$2/$3.git",
-		httpUsernameGuessRepl: "$2",
-		sshRepoGuessRepl:      "git@$1:$2/$3.git",
+		rx: regexp.MustCompile(
+			`\A([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-0-9A-Za-z]+)(\.git)?\z`,
+		),
+		httpRepoGuessRepl: "https://$1/$2/$3.git",
+		sshRepoGuessRepl:  "git@$1:$2/$3.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\A(https?://)([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-0-9A-Za-z]+)(\.git)?\z`),
-		httpRepoGuessRepl:     "$1$2/$3/$4.git",
-		httpUsernameGuessRepl: "$3",
-		sshRepoGuessRepl:      "git@$2:$3/$4.git",
+		rx: regexp.MustCompile(
+			`\A(https?://)([-.0-9A-Za-z]+)/([-0-9A-Za-z]+)/([-0-9A-Za-z]+)(\.git)?\z`,
+		),
+		httpRepoGuessRepl: "$1$2/$3/$4.git",
+		sshRepoGuessRepl:  "git@$2:$3/$4.git",
 	},
 	{
-		rx:                    regexp.MustCompile(`\Asr\.ht/~([-0-9A-Za-z]+)\z`),
-		httpRepoGuessRepl:     "https://git.sr.ht/~$1/dotfiles",
-		httpUsernameGuessRepl: "$1",
-		sshRepoGuessRepl:      "git@git.sr.ht:~$1/dotfiles",
+		rx:                regexp.MustCompile(`\Asr\.ht/~([a-z_][a-z0-9_-]+)\z`),
+		httpRepoGuessRepl: "https://git.sr.ht/~$1/dotfiles",
+		sshRepoGuessRepl:  "git@git.sr.ht:~$1/dotfiles",
 	},
 	{
-		rx:                    regexp.MustCompile(`\Asr\.ht/~([-0-9A-Za-z]+)/([-0-9A-Za-z]+)\z`),
-		httpRepoGuessRepl:     "https://git.sr.ht/~$1/$2",
-		httpUsernameGuessRepl: "$1",
-		sshRepoGuessRepl:      "git@git.sr.ht:~$1/$2",
+		rx:                regexp.MustCompile(`\Asr\.ht/~([a-z_][a-z0-9_-]+)/([-0-9A-Za-z]+)\z`),
+		httpRepoGuessRepl: "https://git.sr.ht/~$1/$2",
+		sshRepoGuessRepl:  "git@git.sr.ht:~$1/$2",
 	},
 }
 
@@ -99,26 +97,53 @@ func (c *Config) newInitCmd() *cobra.Command {
 		Long:    mustLongHelp("init"),
 		Example: example("init"),
 		RunE:    c.runInitCmd,
-		Annotations: map[string]string{
-			modifiesDestinationDirectory: "true",
-			persistentStateMode:          persistentStateModeReadWrite,
-			requiresSourceDirectory:      "true",
-			requiresWorkingTree:          "true",
-			runsCommands:                 "true",
-		},
+		Annotations: newAnnotations(
+			createSourceDirectoryIfNeeded,
+			modifiesDestinationDirectory,
+			persistentStateModeReadWrite,
+			requiresWorkingTree,
+			runsCommands,
+		),
 	}
 
 	flags := initCmd.Flags()
-	flags.BoolVarP(&c.init.apply, "apply", "a", c.init.apply, "update destination directory")
+	c.addInteractiveTemplateFuncFlags(flags)
+	flags.BoolVarP(&c.init.apply, "apply", "a", c.init.apply, "Update destination directory")
+	flags.StringVar(&c.init.branch, "branch", c.init.branch, "Set initial branch to checkout")
 	flags.VarP(&c.init.configPath, "config-path", "C", "Path to write generated config file")
 	flags.BoolVar(&c.init.data, "data", c.init.data, "Include existing template data")
 	flags.IntVarP(&c.init.depth, "depth", "d", c.init.depth, "Create a shallow clone")
-	flags.VarP(c.init.exclude, "exclude", "x", "Exclude entry types")
+	flags.VarP(c.init.filter.Exclude, "exclude", "x", "Exclude entry types")
+	flags.BoolVarP(
+		&c.init.guessRepoURL,
+		"guess-repo-url",
+		"g",
+		c.init.guessRepoURL,
+		"Guess the repo URL",
+	)
+	flags.VarP(c.init.filter.Include, "include", "i", "Include entry types")
 	flags.BoolVar(&c.init.oneShot, "one-shot", c.init.oneShot, "Run in one-shot mode")
-	flags.BoolVarP(&c.init.purge, "purge", "p", c.init.purge, "Purge config and source directories after running")
-	flags.BoolVarP(&c.init.purgeBinary, "purge-binary", "P", c.init.purgeBinary, "Purge chezmoi binary after running")
-	flags.StringVar(&c.init.branch, "branch", c.init.branch, "Set initial branch to checkout")
-	flags.BoolVar(&c.init.ssh, "ssh", false, "Use ssh instead of https when guessing dotfile repo URL")
+	flags.BoolVarP(
+		&c.init.purge,
+		"purge",
+		"p",
+		c.init.purge,
+		"Purge config and source directories after running",
+	)
+	flags.BoolVarP(
+		&c.init.purgeBinary,
+		"purge-binary",
+		"P",
+		c.init.purgeBinary,
+		"Purge chezmoi binary after running",
+	)
+	flags.BoolVar(
+		&c.init.recurseSubmodules,
+		"recurse-submodules",
+		c.init.recurseSubmodules,
+		"Checkout submodules recursively",
+	)
+	flags.BoolVar(&c.init.ssh, "ssh", c.init.ssh, "Use ssh instead of https when guessing repo URL")
 
 	return initCmd
 }
@@ -134,10 +159,7 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 
 	// If we're not in a working tree then init it or clone it.
 	gitDirAbsPath := c.WorkingTreeAbsPath.JoinString(git.GitDirName)
-	switch fileInfo, err := c.baseSystem.Stat(gitDirAbsPath); {
-	case err == nil && fileInfo.IsDir():
-	case err == nil && !fileInfo.IsDir():
-		return fmt.Errorf("%s: not a directory", gitDirAbsPath)
+	switch _, err := c.baseSystem.Stat(gitDirAbsPath); {
 	case errors.Is(err, fs.ErrNotExist):
 		workingTreeRawPath, err := c.baseSystem.RawPath(c.WorkingTreeAbsPath)
 		if err != nil {
@@ -155,15 +177,24 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 				return err
 			}
 		} else {
-			username, dotfilesRepoURL := guessDotfilesRepoURL(args[0], c.init.ssh)
+			var repoURLStr string
+			if c.init.guessRepoURL {
+				repoURLStr = guessRepoURL(args[0], c.init.ssh)
+			} else {
+				repoURLStr = args[0]
+			}
 			if useBuiltinGit {
-				if err := c.builtinGitClone(username, dotfilesRepoURL, workingTreeRawPath); err != nil {
+				if err := c.builtinGitClone(repoURLStr, workingTreeRawPath); err != nil {
 					return err
 				}
 			} else {
 				args := []string{
 					"clone",
-					"--recurse-submodules",
+				}
+				if c.init.recurseSubmodules {
+					args = append(args,
+						"--recurse-submodules",
+					)
 				}
 				if c.init.branch != "" {
 					args = append(args,
@@ -176,7 +207,7 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 					)
 				}
 				args = append(args,
-					dotfilesRepoURL,
+					repoURLStr,
 					workingTreeRawPath.String(),
 				)
 				if err := c.run(chezmoi.EmptyAbsPath, c.Git.Command, args); err != nil {
@@ -188,14 +219,19 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if err := c.createAndReloadConfigFile(); err != nil {
+	if err := c.checkVersion(); err != nil {
+		return err
+	}
+
+	if err := c.createAndReloadConfigFile(cmd); err != nil {
 		return err
 	}
 
 	// Apply.
 	if c.init.apply {
 		if err := c.applyArgs(cmd.Context(), c.destSystem, c.DestDirAbsPath, noArgs, applyArgsOptions{
-			include:      chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll).Sub(c.init.exclude),
+			cmd:          cmd,
+			filter:       c.init.filter,
 			recursive:    false,
 			umask:        c.Umask,
 			preApplyFunc: c.defaultPreApplyFunc,
@@ -205,9 +241,14 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 	}
 
 	// Purge.
-	if c.init.purge {
-		if err := c.doPurge(&purgeOptions{
-			binary: runtime.GOOS != "windows" && c.init.purgeBinary,
+	if c.init.purge || c.init.purgeBinary {
+		if err := c.doPurge(&doPurgeOptions{
+			binary:          c.init.purgeBinary,
+			cache:           c.init.purge,
+			config:          c.init.purge,
+			persistentState: c.init.purge,
+			sourceDir:       c.init.purge,
+			workingTree:     c.init.purge,
 		}); err != nil {
 			return err
 		}
@@ -217,17 +258,26 @@ func (c *Config) runInitCmd(cmd *cobra.Command, args []string) error {
 }
 
 // builtinGitClone clones a repo using the builtin git command.
-func (c *Config) builtinGitClone(username, url string, workingTreeRawPath chezmoi.AbsPath) error {
+func (c *Config) builtinGitClone(repoURLStr string, workingTreeRawPath chezmoi.AbsPath) error {
+	endpoint, err := transport.NewEndpoint(repoURLStr)
+	if err != nil {
+		return err
+	}
+	if c.init.ssh || endpoint.Protocol == "ssh" {
+		return errors.New("builtin git does not support cloning repos over ssh, please install git")
+	}
+
 	isBare := false
 	var referenceName plumbing.ReferenceName
 	if c.init.branch != "" {
 		referenceName = plumbing.NewBranchReferenceName(c.init.branch)
 	}
 	cloneOptions := git.CloneOptions{
-		URL:               url,
+		URL:               repoURLStr,
 		Depth:             c.init.depth,
 		ReferenceName:     referenceName,
 		RecurseSubmodules: git.DefaultSubmoduleRecursionDepth,
+		ShallowSubmodules: c.init.depth == 1,
 	}
 
 	for {
@@ -241,16 +291,13 @@ func (c *Config) builtinGitClone(username, url string, workingTreeRawPath chezmo
 			return err
 		}
 
-		if _, err := fmt.Fprintf(c.stdout, "chezmoi: %s: %v\n", url, err); err != nil {
+		if _, err := fmt.Fprintf(c.stdout, "chezmoi: %s: %v\n", repoURLStr, err); err != nil {
 			return err
 		}
 		var basicAuth http.BasicAuth
-		if basicAuth.Username, err = c.readLine(fmt.Sprintf("Username [default %q]? ", username)); err != nil {
+		if basicAuth.Username, err = c.readString("Username? ", nil); err != nil {
 			return err
 		}
-		if basicAuth.Username == "" {
-			basicAuth.Username = username
-		}
 		if basicAuth.Password, err = c.readPassword("Password? "); err != nil {
 			return err
 		}
@@ -310,22 +357,17 @@ func (o loggableGitCloneOptions) MarshalZerologObject(e *zerolog.Event) {
 	}
 }
 
-// guessDotfilesRepoURL guesses the user's username and dotfile repo from arg.
-func guessDotfilesRepoURL(arg string, ssh bool) (username, repo string) {
-	for _, dotfileRepoGuess := range dotfilesRepoGuesses {
-		if !dotfileRepoGuess.rx.MatchString(arg) {
-			continue
-		}
+// guessRepoURL guesses the user's username and repo from arg.
+func guessRepoURL(arg string, ssh bool) string {
+	for _, repoGuess := range repoGuesses {
 		switch {
-		case ssh && dotfileRepoGuess.sshRepoGuessRepl != "":
-			repo = dotfileRepoGuess.rx.ReplaceAllString(arg, dotfileRepoGuess.sshRepoGuessRepl)
-			return
-		case !ssh && dotfileRepoGuess.httpRepoGuessRepl != "":
-			username = dotfileRepoGuess.rx.ReplaceAllString(arg, dotfileRepoGuess.httpUsernameGuessRepl)
-			repo = dotfileRepoGuess.rx.ReplaceAllString(arg, dotfileRepoGuess.httpRepoGuessRepl)
-			return
+		case !repoGuess.rx.MatchString(arg):
+			continue
+		case ssh && repoGuess.sshRepoGuessRepl != "":
+			return repoGuess.rx.ReplaceAllString(arg, repoGuess.sshRepoGuessRepl)
+		case !ssh && repoGuess.httpRepoGuessRepl != "":
+			return repoGuess.rx.ReplaceAllString(arg, repoGuess.httpRepoGuessRepl)
 		}
 	}
-	repo = arg
-	return
+	return arg
 }
